Package: radiant.multivariate
Type: Package
Title: Multivariate Menu for Radiant: Business Analytics using R and
        Shiny
Version: 1.6.8
Date: 2026-1-9
Authors@R: person("Vincent", "Nijs", , "radiant@rady.ucsd.edu", c("aut", "cre"))
Description: The Radiant Multivariate menu includes interfaces for perceptual
    mapping, factor analysis, cluster analysis, and conjoint analysis. The
    application extends the functionality in 'radiant.data'.
Depends: R (>= 4.3.0), radiant.data (>= 1.6.6)
Imports: radiant.model (>= 1.6.6), shiny (>= 1.8.1), dplyr (>= 1.0.7),
        rlang (>= 0.4.10), ggplot2 (>= 2.2.1), scales (>= 0.4.0),
        magrittr (>= 1.5), psych (>= 1.8.4), GPArotation (>=
        2014.11-1), car (>= 2.1.1), MASS (>= 7.3), import (>= 1.1.0),
        ggrepel (>= 0.8), lubridate (>= 1.7.4), polycor (>= 0.7.10),
        gower (>= 0.2.1), clustMixType (>= 0.2.1), patchwork (>= 1.0.0)
Suggests: testthat (>= 2.0.0), pkgdown (>= 1.1.0)
URL: https://github.com/radiant-rstats/radiant.multivariate/,
        https://radiant-rstats.github.io/radiant.multivariate/,
        https://radiant-rstats.github.io/docs/
BugReports: https://github.com/radiant-rstats/radiant.multivariate/issues/
License: AGPL-3 | file LICENSE
LazyData: true
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-01-10 02:21:31 UTC; vnijs
Author: Vincent Nijs [aut, cre]
Maintainer: Vincent Nijs <radiant@rady.ucsd.edu>
Repository: CRAN
Date/Publication: 2026-01-10 15:10:07 UTC
Built: R 4.6.0; ; 2026-04-25 14:17:37 UTC; unix
