# Generated by roxygen2: do not edit by hand

S3method(plot,conjoint)
S3method(plot,full_factor)
S3method(plot,hclus)
S3method(plot,kclus)
S3method(plot,mds)
S3method(plot,pre_factor)
S3method(plot,prmap)
S3method(predict,conjoint)
S3method(print,conjoint.predict)
S3method(store,conjoint)
S3method(store,conjoint.predict)
S3method(store,full_factor)
S3method(store,hclus)
S3method(store,kclus)
S3method(summary,conjoint)
S3method(summary,full_factor)
S3method(summary,hclus)
S3method(summary,kclus)
S3method(summary,mds)
S3method(summary,pre_factor)
S3method(summary,prmap)
export(clean_loadings)
export(conjoint)
export(full_factor)
export(hclus)
export(kclus)
export(mds)
export(pre_factor)
export(predict_conjoint_by)
export(prmap)
export(radiant.multivariate)
export(radiant.multivariate_viewer)
export(radiant.multivariate_window)
export(the_table)
import(ggplot2)
import(radiant.data)
import(shiny)
importFrom(GPArotation,oblimin)
importFrom(GPArotation,quartimax)
importFrom(GPArotation,simplimax)
importFrom(MASS,isoMDS)
importFrom(car,vif)
importFrom(clustMixType,kproto)
importFrom(dplyr,across)
importFrom(dplyr,bind_rows)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,funs)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_at)
importFrom(dplyr,lag)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_if)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,select_at)
importFrom(dplyr,select_if)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_all)
importFrom(ggrepel,geom_text_repel)
importFrom(gower,gower_dist)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(import,from)
importFrom(lubridate,is.Date)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,"%T>%")
importFrom(magrittr,set_colnames)
importFrom(magrittr,set_names)
importFrom(magrittr,set_rownames)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,wrap_plots)
importFrom(polycor,hetcor)
importFrom(psych,KMO)
importFrom(psych,cortest.bartlett)
importFrom(psych,fa)
importFrom(psych,fa.sort)
importFrom(psych,factor.scores)
importFrom(psych,principal)
importFrom(psych,scoreIrt)
importFrom(radiant.data,launch)
importFrom(radiant.model,predict_model)
importFrom(radiant.model,print_predict_model)
importFrom(rlang,.data)
importFrom(scales,percent)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cmdscale)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,factanal)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(utils,head)
